#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU 16000000
#include <util/delay.h>
#define N 32
#define MAXFREQ 10000
#include "lookup.h"


#define ROWS 6
#define COLUMNS 9

#define MAX(x, y) (((x) > (y)) ? (x) : (y))
#define MIN(x, y) (((x) < (y)) ? (x) : (y))

void adc_init();
uint16_t adc_read();
void TRANSFORM();
void timer1_init();
void timer0_init();

uint8_t lcd_buf1[16];
uint8_t lcd_buf2[16];
int32_t fx[N];
int32_t Fu[N/2][2];

volatile unsigned char ind;
volatile char vector[COLUMNS] = {0};

void clear_column() {
	PORTD &= ~(0b00111111 << PD0);
}

void draw()
{
	unsigned char i;
	unsigned char val;
	if (ind < 8) {
		PORTB &= ~(1 << (PB0 + ind));
	} else {
		PORTD &= ~(1 << PD7);
	}
	clear_column();
	
	ind = (ind + 1) % COLUMNS;
	
	if (ind < 8) {
		PORTB |= (1 << (PB0 + ind));
	} else {
		PORTD |= (1 << PD7);
	}
	val = MIN ( vector[ind], ROWS - 1 );
	for (i = 0 ; i <= val ; i++) {
		PORTD |= (1 << (PD0 + i));
	}
}


ISR(TIMER0_COMPA_vect)
{
	draw();
}

volatile unsigned char indice;

ISR(TIMER1_COMPA_vect)
{
	if (indice < N) {
		fx[indice] = ((int16_t)adc_read());
		indice++;
	}
	else {	
		
		indice = 0;
	}
}


int main()
{
	
	DDRB = 0b11111111;
	DDRD = 0b10111111;
	DDRA = 0b00000000;
	
	PORTB &= 0b00000000;
	PORTD &= 0b01000000;

    int i,j, temp_value;
    uint8_t temp_ind, mag;
    adc_init();

    timer1_init();
	timer0_init();
	
	sei();
	

    while(1) {
	
		if(indice == N) {
		
		TRANSFORM();
		
		for(i =0; i<COLUMNS; i++) {
			j = i + 1;
            if(Fu[j][0]<0)Fu[j][0]*=-1;
            if(Fu[j][1]<0)Fu[j][1]*=-1;
            mag = (uint8_t)(Fu[j][0] + Fu[j][1]) / 4;
			
            if((mag)>3) {
				vector[i] = (mag - 3) / 2 + 3;
            }
            else {
				vector[i] = mag;
            }
        }
		
		}
    }
	return 0;
}


void TRANSFORM()
{
    int16_t count,degree;
    uint8_t u,k;
    count = 0;
    for (u=0; u<N/2; u++) {
        for (k=0; k<N; k++) {
            degree = (uint16_t)pgm_read_byte_near(degree_lookup + count)*2;
            count++;
            Fu[u][0] +=  fx[k] * (int16_t)pgm_read_word_near(cos_lookup + degree);
            Fu[u][1] += -fx[k] * (int16_t)pgm_read_word_near(sin_lookup + degree);
        }
        Fu[u][0] /= N;
        Fu[u][0] /= MAXFREQ;
        Fu[u][1] /= N;
        Fu[u][1] /= MAXFREQ;
    }
}

void timer0_init()
{
	OCR0A = F_CPU / 1024 / 60 / COLUMNS;
	TCCR0A = (1 << WGM01); // CTC comp A
	TCCR0B = (0b101 << CS00); // prescaler 1/1024
	TIMSK0 = (1 << OCIE0A);
}

void timer1_init()
{
    TCCR1B = (1<<WGM12)|(1<<CS10);
    OCR1A = 800;
	TIMSK1 = (1 << OCIE1A);
}

void adc_init()
{
    ADMUX = 0b11000100;
    ADCSRA =0b10000010;
}

uint16_t adc_read()
{
    volatile uint16_t retl,reth;
    ADCSRA |= 1<<ADSC;
    while(!ADIF);
    ADCSRA |= 1<<ADIF;
    retl = ADCL;
    reth = ADCH;
    reth<<=8;
    reth|=retl;
    return reth;
}


